#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.site.interfaces import ISiteContainer, ISiteManager
from pyams_i18n.interfaces import INegotiator, II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import IDataExtractedEvent
from zope.intid.interfaces import IIntIds

# import packages
from pyams_content.shared.site.zmi.widget import SiteManagerFoldersSelectorFieldWidget
from pyams_form.form import AJAXAddForm
from pyams_i18n.schema import I18nTextLineField
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface, Invalid
from zope.schema import Text, Int

from pyams_content import _


@viewlet_config(name='add-site-folder.menu', context=ISiteContainer, layer=IAdminLayer, view=Interface,
                manager=IToolbarAddingMenu, permission=MANAGE_SITE_PERMISSION, weight=10)
class SiteFolderAddMenu(ToolbarMenuItem):
    """Site folder add menu"""

    label = _("Add site folder...")
    label_css_class = 'fa fa-fw fa-folder-o'
    url = 'add-site-folder.html'
    modal_target = True


class ISiteFolderAddFormFields(Interface):
    """Site folder add form fields interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Visible label used to display content"),
                              required=True)

    parent = Int(title=_("Parent"),
                 description=_("Folder's parent"),
                 required=True)

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)


@pagelet_config(name='add-site-folder.html', context=ISiteContainer, layer=IPyAMSLayer,
                permission=MANAGE_SITE_PERMISSION)
class SiteFolderAddForm(AdminDialogAddForm):
    """Site folder add form"""

    @property
    def title(self):
        return II18n(self.context).query_attribute('title', request=self.request)

    legend = _("Add site folder")
    icon_css_class = 'fa fa-fw fa-folder-o'

    fields = field.Fields(ISiteFolderAddFormFields)
    fields['parent'].widgetFactory = SiteManagerFoldersSelectorFieldWidget

    ajax_handler = 'add-site-folder.json'
    edit_permission = MANAGE_SITE_PERMISSION

    def updateWidgets(self, prefix=None):
        super(SiteFolderAddForm, self).updateWidgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = MANAGE_SITE_PERMISSION
        if 'notepad' in self.widgets:
            self.widgets['notepad'].widget_css_class = 'textarea'

    def create(self, data):
        manager = get_parent(self.context, ISiteManager)
        return manager.folder_factory()

    def update_content(self, content, data):
        content.title = data['title']
        content.short_name = data['title']
        content.notepad = data['notepad']
        intids = get_utility(IIntIds)
        parent = intids.queryObject(data.get('parent'))
        if parent is not None:
            negotiator = get_utility(INegotiator)
            title = II18n(content).get_attribute('title', lang=negotiator.server_language)
            name = translate_string(title, force_lower=True, spaces='-')
            if name in parent:
                index = 1
                new_name = '{name}-{index:02}'.format(name=name, index=index)
                while new_name in parent:
                    index += 1
                    new_name = '{name}-{index:02}'.format(name=name, index=index)
                name = new_name
            parent[name] = content

    def add(self, content):
        pass

    def nextURL(self):
        return absolute_url(self.context, self.request, 'admin#site-tree.html')


@subscriber(IDataExtractedEvent, form_selector=SiteFolderAddForm)
def handle_site_folder_add_form_data_extraction(event):
    """Handle site folder add form data extraction"""
    negotiator = get_utility(INegotiator)
    title = event.data.get('title', {}).get(negotiator.server_language)
    if not title:
        event.form.widgets.errors += (Invalid(_("You must provide a folder name for default server language!")),)


@view_config(name='add-site-folder.json', context=ISiteContainer, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_PERMISSION, renderer='json', xhr=True)
class SiteFolderAJAXAddForm(AJAXAddForm, SiteFolderAddForm):
    """Site folder add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload'}
